"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Device = void 0;
const winix_api_1 = require("winix-api");
const async_lock_1 = __importDefault(require("async-lock"));
class Device {
    constructor(deviceId, cacheIntervalMs, log) {
        this.deviceId = deviceId;
        this.cacheIntervalMs = cacheIntervalMs;
        this.log = log;
        this.lastWinixPoll = -1;
        this.lock = new async_lock_1.default({ timeout: 3000 });
        this.state = {
            power: winix_api_1.Power.Off,
            mode: winix_api_1.Mode.Auto,
            airflow: winix_api_1.Airflow.Low,
            airQuality: winix_api_1.AirQuality.Good,
            plasmawave: winix_api_1.Plasmawave.Off,
            ambientLight: 0,
            filterHours: 0,
        };
    }
    hasData() {
        return this.lastWinixPoll > -1;
    }
    async getPower() {
        await this.ensureUpdated();
        return this.state.power;
    }
    async getMode() {
        await this.ensureUpdated();
        return this.state.mode;
    }
    async getAirflow() {
        await this.ensureUpdated();
        return this.state.airflow;
    }
    async getAirQuality() {
        await this.ensureUpdated();
        return this.state.airQuality;
    }
    async getPlasmawave() {
        await this.ensureUpdated();
        return this.state.plasmawave;
    }
    async getAmbientLight() {
        await this.ensureUpdated();
        return this.state.ambientLight;
    }
    async getFilterHours() {
        await this.ensureUpdated();
        return this.state.filterHours;
    }
    async setPower(value) {
        const initialPower = await this.getPower();
        if (initialPower === value) {
            this.log.debug('device:setPower(%s)', value, '(no change)');
            return;
        }
        this.log.debug('device:setPower()', initialPower, value);
        await winix_api_1.WinixAPI.setPower(this.deviceId, value);
        this.state.power = value;
        // default to auto mode when turning on from off
        if (initialPower === winix_api_1.Power.Off && value === winix_api_1.Power.On) {
            this.state.mode = winix_api_1.Mode.Auto;
        }
    }
    async setMode(value) {
        const turnedOn = await this.ensureOn();
        // Don't try to set the mode if it's already set to the same value
        // Fixes issues with this being set right around the time of power on
        if (!turnedOn && value === await this.getMode()) {
            this.log.debug('device:setMode(%s)', value, '(no change)');
            return;
        }
        this.log.debug('device:setMode(%s)', value);
        await winix_api_1.WinixAPI.setMode(this.deviceId, value);
        this.state.mode = value;
        // default to low airflow when switching modes
        this.state.airflow = winix_api_1.Airflow.Low;
    }
    async setAirflow(value) {
        this.log.debug('device:setAirflow(%s)', value);
        // Device must be on and in manual mode to set airflow
        await this.ensureOn();
        await this.setMode(winix_api_1.Mode.Manual);
        await winix_api_1.WinixAPI.setAirflow(this.deviceId, value);
        this.state.airflow = value;
    }
    async setPlasmawave(value) {
        this.log.debug('device:setPlasmawave()', value);
        await this.ensureOn();
        await winix_api_1.WinixAPI.setPlasmawave(this.deviceId, value);
        this.state.plasmawave = value;
    }
    async getState() {
        await this.ensureUpdated();
        return {
            power: this.state.power,
            mode: this.state.mode,
            airflow: this.state.airflow,
            airQuality: this.state.airQuality,
            plasmawave: this.state.plasmawave,
            ambientLight: this.state.ambientLight,
            filterHours: this.state.filterHours,
        };
    }
    async update() {
        this.log.debug('device:update()');
        this.state = await winix_api_1.WinixAPI.getDeviceStatus(this.deviceId);
        this.log.debug('device:update()', JSON.stringify(this.state));
        this.lastWinixPoll = Date.now();
    }
    async ensureUpdated() {
        // Use a lock to ensure only one update is running at a time
        await this.lock.acquire('update', async () => {
            if (this.shouldUpdate()) {
                await this.update();
            }
        });
    }
    shouldUpdate() {
        return Date.now() - this.lastWinixPoll > this.cacheIntervalMs;
    }
    async ensureOn() {
        if (await this.getPower() === winix_api_1.Power.On) {
            this.log.debug('device:ensureOn()', 'already on');
            return false;
        }
        this.log.debug('device:ensureOn()');
        await this.setPower(winix_api_1.Power.On);
        return true;
    }
}
exports.Device = Device;
